@@ -1,4 +1,5 @@
 /* ACME
+ * init_tray (acme);
  * Copyright (C) 2001 Bastien Nocera <hadess@hadess.net>
  *
  * acme.c
@@ -19,6 +20,8 @@
  * USA.
  */
 
+#include <config.h>
+
 /* system headers */
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -31,10 +34,10 @@
 #include <gnome.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
+#include "eggtrayicon.h"
 
-#include "config.h"
 #include "acme.h"
-#include "volume-oss.h"
+#include "g-volume.h"
 #ifdef USE_FBLEVEL
 #include "fb-level.h"
 #endif
@@ -43,7 +46,7 @@ static gboolean event_cb (GtkWidget * widget, GdkEvent * event,
 			  gpointer data);
 
 typedef struct {
-	VolumeOSS *volobj;
+	GVolume *volobj;
 #ifdef USE_FBLEVEL
 	FBLevel *levobj;
 #endif
@@ -53,7 +56,11 @@ typedef struct {
 	int NumLockMask, CapsLockMask, ScrollLockMask;
 	GConfClient *conf_client;
 	guint dialog_timeout;
-	int lock_fd;
+
+	/* Tray icon */
+	EggTrayIcon *tray_icon;
+	GtkTooltips *tray_icon_tooltip;
+	GtkWidget *popup_menu;
 } Acme;
 
 enum {
@@ -64,63 +71,54 @@ enum {
 };
 
 static void
+selection_get_func (GtkClipboard *clipboard, GtkSelectionData *selection_data,
+		guint info, gpointer user_data_or_owner)
+{
+}
+
+static void
+selection_clear_func (GtkClipboard *clipboard, gpointer user_data_or_owner)
+{       
+	return;
+}
+
+#define SELECTION_NAME "_ACME_SELECTION"
+
+static gboolean
 acme_get_lock (Acme *acme)
 {
-	char *path;
-	int fd;
+	gboolean result = FALSE;
+	GtkClipboard *clipboard;
+	Atom clipboard_atom = gdk_x11_get_xatom_by_name (SELECTION_NAME);
+	static const GtkTargetEntry targets[] = {
+		{ SELECTION_NAME, 0, 0 }
+	};
 
-	acme->lock_fd = -1;
+	XGrabServer (GDK_DISPLAY());
 
-	path = g_strdup_printf ("%s/.acme.lock", g_get_home_dir());
-	fd = open (path, O_WRONLY | O_CREAT, S_IRWXU);
-#ifdef DEBUG
-	g_message ("opened %s, got fd %d", path, fd);
-#endif
+	if (XGetSelectionOwner (GDK_DISPLAY(), clipboard_atom) != None)
+		goto out;
 
-	g_free (path);
-	if (fd < 0)
-	{
-#ifdef DEBUG
-		g_message ("Couldn't open the lock file");
-#endif
-		return;
-	}
+	clipboard = gtk_clipboard_get (gdk_atom_intern (SELECTION_NAME, FALSE));
 
-	if (lockf (fd, F_TEST, 0L) < 0)
-	{
-#ifdef DEBUG
-		g_message ("Couldn't get lock on file");
-#endif
-		close (fd);
-		return;
-	}
+	if (!gtk_clipboard_set_with_data  (clipboard, targets,
+				G_N_ELEMENTS (targets),
+				selection_get_func,
+				selection_clear_func, NULL))
+		goto out;
 
-	if (lockf (fd, F_TLOCK, 0L) < 0)
-	{
-#ifdef DEBUG
-		g_message ("for some reason, we couldn't get the lock");
-#endif
-		close (fd);
-		return;
-	}
+	result = TRUE;
 
-	acme->lock_fd = fd;
-	return;
+out:
+	XUngrabServer (GDK_DISPLAY());
+	gdk_flush();
+
+	return result;
 }
 
 static void
 acme_exit (Acme *acme)
 {
-	if (acme->lock_fd >= 0)
-	{
-		char *path;
-
-		close (acme->lock_fd);
-		path = g_strdup_printf ("%s/.acme.lock", g_get_home_dir());
-		unlink (path);
-		g_free (path);
-	}
-
 	exit (0);
 }
 
@@ -139,12 +137,21 @@ acme_error (char * msg)
 			GTK_RESPONSE_OK);
 	gtk_widget_show (error_dialog);
 	gtk_dialog_run (GTK_DIALOG (error_dialog));
+	gtk_widget_destroy (error_dialog);
 }
 
 static void
-execute (char * cmd)
+execute (char *cmd, gboolean sync)
 {
-	if (g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL) == FALSE)
+	gboolean retval;
+
+	if (sync == TRUE)
+		retval = g_spawn_command_line_sync
+			(cmd, NULL, NULL, NULL, NULL);
+	else
+		retval = g_spawn_command_line_async (cmd, NULL);
+
+	if (retval == FALSE)
 	{
 		char *msg;
 
@@ -155,8 +162,28 @@ execute (char * cmd)
 
 		acme_error (msg);
 		g_free (msg);
+	}
+}
 
-		exit (1);
+static void
+execute_this_or_that (char *cmd1, char *cmd2)
+{
+	if (g_spawn_command_line_async (cmd1, NULL) == FALSE)
+	{
+		if (g_spawn_command_line_async (cmd2, NULL) == FALSE)
+		{
+			char *msg;
+
+			msg = g_strdup_printf
+				(_("Couldn't execute either command: %s\n"
+				   "or command: %s\n"
+				   "Verify that at least one of these commands"
+				   "exist."),
+				 cmd1, cmd2);
+
+			acme_error (msg);
+			g_free (msg);
+		}
 	}
 }
 
@@ -184,27 +211,44 @@ acme_play_sound (Acme *acme)
 		return;
 	}
 
-	execute (command);
+	execute (command, FALSE);
 	g_free (command);
 }
 
-static gboolean
-vol_problem_cb (VolumeOSS *fb, gpointer data)
+static char*
+permission_problem_string (const char *files)
 {
-	acme_error(_("Permissions on the file /dev/mixer are broken\n"
-		"Please check ACME's documentation, correct the problem "
-		"and restart ACME."));
+	return g_strdup_printf (_("Permissions on the file %s are broken\n"
+				"Please check ACME's documentation, correct "
+				"the problem and restart ACME."), files);
+}
+
+static gboolean
+vol_problem_cb (GVolume *fb, gpointer data)
+{
+	char *msg;
+
+	msg = permission_problem_string ("/dev/mixer");
+	acme_error (msg);
+	g_free (msg);
+
 	return TRUE;
 }
 
 #ifdef USE_FBLEVEL
-static gboolean
-fb_problem_cb (FBLevel *fb, gpointer data)
+static void
+fb_problem_cb (void)
 {
-	acme_error(_("Permissions on the files /dev/fb0 and/or /dev/pmu "
-		"are broken.\nPlease check ACME's documentation, correct "
-		"the problem and restart ACME."));
-	return TRUE;
+	char *msg;
+
+	if (fb_level_is_powerbook () == FALSE)
+		return;
+
+	msg = permission_problem_string ("/dev/pmu");
+	acme_error (msg);
+	g_free (msg);
+
+	return;
 }
 #endif
 
@@ -218,7 +262,7 @@ update_use_pcm_cb (GConfClient *client, guint id, GConfEntry *entry,
 	use_pcm = gconf_client_get_bool (acme->conf_client,
 			"/apps/acme/use_pcm",
 			NULL);
-	volume_oss_set_use_pcm (acme->volobj, use_pcm);
+	g_volume_set_use_pcm (acme->volobj, use_pcm);
 }
 
 static void
@@ -275,18 +319,50 @@ static void
 grab_key (int key_code)
 {
 	gdk_error_trap_push ();
-	XGrabKey (GDK_DISPLAY (), key_code, AnyModifier,
-		  GDK_ROOT_WINDOW (), True,
-		  GrabModeAsync, GrabModeAsync);
+
+	XGrabKey (GDK_DISPLAY (), key_code,
+			0,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod2Mask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod5Mask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			LockMask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod2Mask | LockMask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod5Mask | LockMask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod2Mask | Mod5Mask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod2Mask | Mod5Mask | LockMask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+
 	gdk_flush ();
-	if (gdk_error_trap_pop ()) {
+	if (gdk_error_trap_pop ())
+	{
 		char *error;
 
 		error = g_strdup_printf
 			(_("It seems that another application already has"
 			   " access to the multimedia keys.\n"
 			   "Key %d couldn't be bound.\n"
-			   "Close this application and restart ACME."),
+			   "Is another daemon already running ?"),
 			 key_code);
 		acme_error (error);
 		g_free (error);
@@ -303,7 +379,8 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 	g_return_if_fail (entry->key != NULL);
 
 	/* Find the key that was modified */
-	for (i = 0; i < HANDLED_KEYS; i++) {
+	for (i = 0; i < HANDLED_KEYS; i++)
+	{
 		if (strcmp (entry->key, keys[i].key_config) == 0)
 		{
 			int key_code;
@@ -312,7 +389,8 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 			key_code = gconf_client_get_int (acme->conf_client,
 					keys[i].key_config,
 					NULL);
-			if (key_code > 0) {
+			if (key_code > 0)
+			{
 				grab_key (key_code);
 				keys[i].key_code = key_code;
 			}
@@ -320,27 +398,32 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 	}
 }
 
-static void
+static gboolean
 init_kbd (Acme *acme)
 {
 	GtkWidget *widget = acme->invisible;
 	int i;
+	gboolean retval = FALSE;
 
 	gtk_widget_show (widget);
 
 	/* Make the root window send events to the invisible proxy widget */
-	//FIXME replace by Gtk+ 2 version of the call
-	gdk_window_set_user_data (GDK_ROOT_PARENT (), widget);
+	gdk_window_set_user_data (gdk_get_default_root_window (), widget);
 
-	XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW (), KeyPressMask);
+	gdk_window_set_events (gdk_get_default_root_window (),
+			GDK_KEY_PRESS_MASK);
 
-	for (i = 0; i < HANDLED_KEYS; i++) {
+	for (i = 0; i < HANDLED_KEYS; i++)
+	{
 		int tmp;
 
 		tmp = gconf_client_get_int (acme->conf_client,
 				keys[i].key_config,
 				NULL);
-		if (tmp != -1)
+
+		keys[i].key_code = tmp;
+
+		if (tmp > 0)
 		{
 #ifdef DEBUG
 			g_print ("grabbed key %d for gconf key %s\n",
@@ -348,16 +431,127 @@ init_kbd (Acme *acme)
 					keys[i].key_config);
 #endif
 			grab_key (keys [i].key_code);
+			retval = TRUE;
 		}
-		keys[i].key_code = tmp;
 	}
 
-	for (i = 0; i < HANDLED_KEYS; i++) {
+	for (i = 0; i < HANDLED_KEYS; i++)
+	{
 		gconf_client_notify_add (acme->conf_client,
 				keys[i].key_config,
 				update_kbd_cb,
 				acme, NULL, NULL);
 	}
+
+	return retval;
+}
+
+static void
+prefs_activated (GtkMenuItem *menuitem, gpointer user_data)
+{
+	execute ("acme-properties", FALSE);
+}
+
+static void
+about_activated (GtkMenuItem *menuitem, gpointer user_data)
+{
+	static GtkWidget *about = NULL;
+	GdkPixbuf *pixbuf = NULL;
+	const gchar *authors[] = { "Bastien Nocera <hadess@hadess.net>", NULL };
+	const gchar *documenters[] = { NULL };
+	const gchar *translator_credits = _("translator_credits");
+
+	if (about != NULL)
+	{
+		gdk_window_raise (about->window);
+		gdk_window_show (about->window);
+		return;
+	}
+
+	pixbuf = gdk_pixbuf_new_from_file (ACME_DATA "acme-48.png", NULL);
+
+	about = gnome_about_new(_("Acme"), VERSION,
+			_("(C) 2001, 2002 Bastien Nocera"),
+			_("Multimedia keys daemon"),
+			(const char **)authors,
+			(const char **)documenters,
+			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+			pixbuf);
+
+	g_signal_connect (G_OBJECT (about), "destroy", G_CALLBACK
+			(gtk_widget_destroyed), &about);
+	g_object_add_weak_pointer (G_OBJECT (about),
+			(void**)&(about));
+
+	gtk_widget_show(about);
+}
+
+static gboolean
+tray_icon_release (GtkWidget *widget, GdkEventButton *event, Acme *acme)
+{
+	if (event->button == 3)
+	{
+		gtk_menu_popdown (GTK_MENU (acme->popup_menu));
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
+static gboolean
+tray_icon_press (GtkWidget *widget, GdkEventButton *event, Acme *acme)
+{
+	if (event->button == 3)
+	{
+		gtk_menu_popup (GTK_MENU (acme->popup_menu), NULL, NULL, NULL,
+				NULL, event->button, event->time);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static void
+init_tray (Acme *acme)
+{
+	GtkWidget *image, *evbox, *item;
+
+	acme->tray_icon = egg_tray_icon_new ("Multimedia Keys daemon");
+	image = gtk_image_new_from_file (ACME_DATA "acme-16.png");
+
+	acme->tray_icon_tooltip = gtk_tooltips_new ();
+	gtk_tooltips_set_tip (acme->tray_icon_tooltip,
+			GTK_WIDGET (acme->tray_icon),
+			_("Multimedia Keys daemon active"),
+			NULL);
+
+	/* Event box */
+	evbox = gtk_event_box_new ();
+	g_signal_connect (G_OBJECT (evbox), "button_press_event",
+			G_CALLBACK (tray_icon_press), (gpointer) acme);
+	g_signal_connect (G_OBJECT (evbox), "button_release_event",
+			G_CALLBACK (tray_icon_release), (gpointer) acme);
+
+	/* Popup menu */
+	acme->popup_menu = gtk_menu_new ();
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PREFERENCES,
+			NULL);
+	g_signal_connect (G_OBJECT (item), "activate",
+			G_CALLBACK (prefs_activated), (gpointer) acme);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (acme->popup_menu), item);
+
+	item = gtk_image_menu_item_new_from_stock (GNOME_STOCK_ABOUT,
+			NULL);
+	g_signal_connect (G_OBJECT (item), "activate",
+			G_CALLBACK (about_activated), (gpointer) acme);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (acme->popup_menu), item);
+
+	gtk_container_add (GTK_CONTAINER (evbox), image);
+	gtk_container_add (GTK_CONTAINER (acme->tray_icon), evbox);
+	gtk_widget_show_all (GTK_WIDGET (acme->tray_icon));
 }
 
 static void
@@ -387,9 +581,7 @@ static gboolean
 dialog_hide (Acme *acme)
 {
 	gtk_widget_hide (acme->dialog);
-
 	acme->dialog_timeout = 0;
-
 	return FALSE;
 }
 
@@ -419,7 +611,25 @@ dialog_show (Acme *acme)
 
 	acme->dialog_timeout = gtk_timeout_add (DIALOG_TIMEOUT,
 			(GtkFunction) dialog_hide, acme);
+}
 
+static void
+do_www_action (Acme *acme)
+{
+	char *string, *command;
+
+	string = gconf_client_get_string (acme->conf_client,
+		"/desktop/gnome/url-handlers/unknown/command",
+		 NULL);
+
+	if (string == NULL && strcmp (string, "") == 0)
+		return;
+
+	command = g_strdup_printf (string, "about:blank");
+	execute (command, FALSE);
+
+	g_free (command);
+	g_free (string);
 }
 
 static void
@@ -460,10 +670,10 @@ do_eject_action (Acme *acme)
 
 	command = gconf_client_get_string (acme->conf_client,
 			"/apps/acme/eject_command", NULL);
-	if ((command != NULL) && (strcmp (command, "") == 0))
-		execute (command);
+	if ((command != NULL) && (strcmp (command, "") != 0))
+		execute (command, TRUE);
 	else
-		execute ("eject");
+		execute ("eject", TRUE);
 
 	gtk_widget_set_sensitive (progress, TRUE);
 }
@@ -515,33 +725,35 @@ do_sound_action (Acme *acme, int type)
 		acme->dialog_timeout = 0;
 	}
 
-	vol = volume_oss_get_volume (acme->volobj);
-	muted = volume_oss_get_mute (acme->volobj);
+	vol = g_volume_get_volume (acme->volobj);
+	muted = g_volume_get_mute (acme->volobj);
 
 	switch (type) {
 	case MUTE_KEY:
-		volume_oss_mute_toggle(acme->volobj);
+		g_volume_mute_toggle(acme->volobj);
 		break;
 	case VOLUME_DOWN_KEY:
-		if (muted) {
-			volume_oss_mute_toggle(acme->volobj);
+		if (muted)
+		{
+			g_volume_mute_toggle(acme->volobj);
 		} else {
-			volume_oss_set_volume (acme->volobj, vol - VOLUME_STEP);
+			g_volume_set_volume (acme->volobj, vol - VOLUME_STEP);
 		}
 		break;
 	case VOLUME_UP_KEY:
-		if (muted) {
-			volume_oss_mute_toggle(acme->volobj);
+		if (muted)
+		{
+			g_volume_mute_toggle(acme->volobj);
 		} else {
-			volume_oss_set_volume (acme->volobj, vol + VOLUME_STEP);
+			g_volume_set_volume (acme->volobj, vol + VOLUME_STEP);
 		}
 		break;
 	}
 
-	muted = volume_oss_get_mute(acme->volobj);
+	muted = g_volume_get_mute(acme->volobj);
 	acme_image_set (acme, muted ? ICON_MUTED : ICON_LOUD);
 
-	vol = volume_oss_get_volume (acme->volobj);
+	vol = g_volume_get_volume (acme->volobj);
 	progress = glade_xml_get_widget (acme->xml, "progressbar");
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress),
 			(double) vol / 100);
@@ -563,19 +775,50 @@ do_action (int type, Acme *acme)
 	case MUTE_KEY:
 	case VOLUME_DOWN_KEY:
 	case VOLUME_UP_KEY:
-		do_sound_action (acme, type);
+		if (acme->volobj != NULL)
+			do_sound_action (acme, type);
 		break;
 	case POWER_KEY:
 		do_exit_action (acme);
 		break;
 	case EJECT_KEY:
-		/* Do nothing, handled differently */
+		do_eject_action (acme);
+		break;
+	case HOME_KEY:
+		execute ("nautilus", FALSE);
+		break;
+	case SEARCH_KEY:
+		execute ("gnome-search-tool", FALSE);
+		break;
+	case EMAIL_KEY:
+		execute_this_or_that ("evolution mailto:", "balsa");
+		break;
+	case SLEEP_KEY:
+		execute ("apm", TRUE);
+		break;
+	case SCREENSAVER_KEY:
+		execute ("xscreensaver-command -lock", FALSE);
+		break;
+	case FINANCE_KEY:
+		execute ("gnucash", FALSE);
+		break;
+	case HELP_KEY:
+		execute ("yelp", FALSE);
+		break;
+	case WWW_KEY:
+		do_www_action (acme);
+		break;
+	case CALCULATOR_KEY:
+		execute ("gnome-calculator", FALSE);
+		break;
+	case RECORD_KEY:
+		execute ("gnome-sound-recorder", FALSE);
 		break;
 #ifdef USE_FBLEVEL
 	case BRIGHT_DOWN_KEY:
 	case BRIGHT_UP_KEY:
-
-		do_brightness_action (acme, type);
+		if (acme->levobj != NULL)
+			do_brightness_action (acme, type);
 		break;
 #endif
 	default:
@@ -583,33 +826,6 @@ do_action (int type, Acme *acme)
 	}
 }
 
-static void
-check_eject (Acme *acme, GdkEvent *event)
-{
-	GdkEventKey *key;
-	static guint eject_press_time = 0;
-
-	key = (GdkEventKey *) event;
-
-#ifdef DEBUG
-	g_print ("type %d (press %d release %d)\n", event->type, GDK_KEY_PRESS, GDK_KEY_RELEASE);
-	if (event->type == GDK_KEY_PRESS || event->type == GDK_KEY_RELEASE)
-		g_print ("key pressed: %d\n", key->hardware_keycode);
-#endif
-
-	if (event->type == GDK_KEY_PRESS
-			&& key->hardware_keycode == keys[EJECT_KEY].key_code
-			&& eject_press_time == 0)
-		eject_press_time = key->time;
-	if (event->type == GDK_KEY_RELEASE
-			&& key->hardware_keycode == keys[EJECT_KEY].key_code)
-	{
-		if (key->time - eject_press_time >= EJECT_TIMEOUT)
-			do_eject_action (acme);
-		eject_press_time = 0;
-	}
-}
-
 static gboolean
 event_cb (GtkWidget * widget, GdkEvent * event, gpointer data)
 {
@@ -617,10 +833,7 @@ event_cb (GtkWidget * widget, GdkEvent * event, gpointer data)
 	GdkEventKey *key;
 	int i;
 
-	/* Eject is handled differently */
-	check_eject (acme, event);
-
-	if (event->type != GDK_KEY_PRESS)
+	if (event->type != GDK_KEY_RELEASE)
 		return TRUE;
 
 	key = (GdkEventKey *) event;
@@ -643,23 +856,31 @@ main (int argc, char *argv[])
 {
 	Acme *acme;
 
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+
 	gnome_program_init ("acme", VERSION,
 			LIBGNOMEUI_MODULE,
 			argc, argv,
 			NULL);
 
-	acme = g_new (Acme, 1);
+	acme = g_new0 (Acme, 1);
+	acme->conf_client = NULL;
 
-	acme_get_lock (acme);
-	if (acme->lock_fd < 0)
+	if (acme_get_lock (acme) == FALSE)
+	{
+		g_print ("Daemon already running, exiting...\n");
 		acme_exit (acme);
+	}
 
 	glade_gnome_init ();
 	acme->xml = glade_xml_new (ACME_DATA "acme.glade", NULL, NULL);
 
 	if (acme->xml == NULL) {
 		acme_error (_("Couldn't load the Glade file.\n"
-				"Make sure that ACME is properly installed."));
+				"Make sure that this daemon is properly"
+				" installed."));
 		exit (1);
 	}
 
@@ -673,35 +894,43 @@ main (int argc, char *argv[])
 			NULL);
 
 	acme->invisible = gtk_invisible_new ();
-	init_kbd (acme);
+	if (init_kbd (acme) == FALSE)
+	{
+		execute ("acme-properties", FALSE);
+		exit (0);
+	}
 	init_sm (acme);
-	gtk_widget_realize(acme->dialog);
+	init_tray (acme);
+	gtk_widget_realize (acme->dialog);
 	acme->dialog_timeout = 0;
 
 	/* initialise Volume handler */
-	acme->volobj = volume_oss_new();
-	g_signal_connect(G_OBJECT(acme->volobj),
-			"fd_problem",
-			(GCallback) vol_problem_cb, NULL);
-	volume_oss_set_use_pcm (acme->volobj,
-			gconf_client_get_bool (acme->conf_client,
-				"/apps/acme/use_pcm", NULL));
-	gconf_client_notify_add (acme->conf_client,
-			"/apps/acme/use_pcm",
-			update_use_pcm_cb,
-			acme, NULL, NULL);
+	acme->volobj = g_volume_new();
+	if (acme->volobj != NULL)
+	{
+		g_signal_connect(G_OBJECT(acme->volobj),
+				"fd_problem",
+				G_CALLBACK (vol_problem_cb), NULL);
+
+		g_volume_set_use_pcm (acme->volobj,
+				gconf_client_get_bool (acme->conf_client,
+					"/apps/acme/use_pcm", NULL));
+		gconf_client_notify_add (acme->conf_client,
+				"/apps/acme/use_pcm",
+				update_use_pcm_cb,
+				acme, NULL, NULL);
+	}
 
 #ifdef USE_FBLEVEL
 	/* initialise Frame Buffer level handler */
 	acme->levobj = fb_level_new();
-	g_signal_connect(G_OBJECT(acme->levobj),
-			"fd_problem",
-			(GCallback) fb_problem_cb, NULL);
+	if (acme->levobj == NULL)
+		fb_problem_cb ();
 #endif
 
 	g_signal_connect (GTK_OBJECT (acme->invisible),
 			    "event",
-			    (GCallback) event_cb, acme);
+			    G_CALLBACK (event_cb), acme);
 
 	gtk_main ();
 
